package eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.text.Position;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

import com.swtdesigner.SWTResourceManager;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionPopulatePropertySheet;
import eu.etaxonomy.taxeditor.prototype2.controller.TaxonTransfer;
import eu.etaxonomy.taxeditor.prototype2.view.NameEditorView;
import eu.etaxonomy.taxeditor.prototype2.view.NameViewer;

public class NameComposite extends Composite {
	
	public static final int ACCEPTED = 0;
	public static final int HOMOTYPIC_SYNONYM = 1;
	public static final int HETEROTYPIC_SYNONYM = 2;
	
	Label relationLabel;
	Synonym synonym;
	Taxon taxon;
	int compositeType;
	NameViewer nameViewer;
	boolean initialized = false;

	/**
	 * Temporary procedure until ability to get accepted taxon
	 *  from synonym is implemented
	 *  
	 * @param parent
	 * @param synonym
	 * @param taxon
	 */
	public NameComposite(Composite parent, Synonym synonym, Taxon taxon) {
		super(parent, SWT.NONE);
		
		this.taxon = taxon;
		this.synonym = synonym;

		// Either initialize group composite, 
		//  or assume its homotypic group
		if (parent instanceof GroupComposite) {
			
			if (((GroupComposite) parent).isAcceptedGroup())
				this.compositeType = HOMOTYPIC_SYNONYM;
			else
				this.compositeType = HETEROTYPIC_SYNONYM;			
			
			if (((GroupComposite) parent).getHomotypicalGroup() == null) 
				((GroupComposite) parent).setHomotypicalGroup(
						synonym.getName().getHomotypicalGroup());
			else
				synonym.getName().setHomotypicalGroup(
						((GroupComposite) parent).getHomotypicalGroup());
		}		
		
		createNameComposite();		
	}
	
	public NameComposite(Composite parent, Synonym synonym) {
		super(parent, SWT.NONE);
		 
		this.synonym = synonym;
//		this.taxon = (Taxon) synonym.getAcceptedTaxa().toArray()[0];

		// Either initialize group composite, 
		//  or assume its homotypic group
		if (parent instanceof GroupComposite) {
			
			if (((GroupComposite) parent).isAcceptedGroup())
				this.compositeType = HOMOTYPIC_SYNONYM;
			else
				this.compositeType = HETEROTYPIC_SYNONYM;
			
			if (((GroupComposite) parent).getHomotypicalGroup() == null) 
				((GroupComposite) parent).setHomotypicalGroup(
						synonym.getName().getHomotypicalGroup());
			else
				synonym.getName().setHomotypicalGroup(
						((GroupComposite) parent).getHomotypicalGroup());
		}				
		
		createNameComposite();
	}

	public NameComposite(Composite parent, Taxon taxon) {
		super(parent, SWT.NONE);
				
		this.taxon = taxon;
		this.compositeType = ACCEPTED;
		
		// Initialize homotypic group component
		if (parent instanceof GroupComposite) {
						
			((GroupComposite) parent).setHomotypicalGroup(
					taxon.getName().getHomotypicalGroup());
			((GroupComposite) parent).setAcceptedGroup(true);
		}
			
		createNameComposite();
	}
	
	/**
	 * 
	 */
	public void createNameComposite() {

		Assert.isNotNull(taxon);
		
		// Graphic layout of composite
		GridLayout gridLayout = new GridLayout();
		gridLayout.verticalSpacing = 0;
		gridLayout.marginWidth = 5;
		gridLayout.marginHeight = 5;
		gridLayout.horizontalSpacing = 5;
		gridLayout.numColumns = (compositeType == ACCEPTED? 1: 2);
		this.setLayout(gridLayout);
		this.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		this.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		if (compositeType != ACCEPTED) {
//			relationLabel.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/heterosyn_no_bg.gif"));	
			relationLabel = new Label(this, SWT.NONE);
			relationLabel.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false));					
			relationLabel.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
			
			if (this.compositeType == HOMOTYPIC_SYNONYM)
				relationLabel.setImage(SWTResourceManager.getImage(NameComposite.class, "homosyn_no_bg.gif"));
//				relationLabel.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/homosyn_no_bg.gif"));
			else
				relationLabel.setImage(SWTResourceManager.getImage(NameComposite.class, "heterosyn_no_bg.gif"));
//				relationLabel.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/heterosyn_no_bg.gif"));
		}
		nameViewer = new NameViewer(this);	
		nameViewer.getTextWidget().setLayoutData(
				new GridData(SWT.FILL, SWT.CENTER, true, false));
		nameViewer.getTextWidget().setFont(
				SWTResourceManager.getFont("Georgia", 
						(compositeType == ACCEPTED? 12 : 10), 
						SWT.NONE));	
		
		// Add verify listener to check for line breaks
		nameViewer.getTextWidget().addVerifyListener(
				new NameViewerVerifyListener(taxon));
		
		// Listen for changes to name
		nameViewer.getTextWidget().addModifyListener(new ModifyListener () {
			public void modifyText(ModifyEvent e) {
				if (initialized) {
					String text = ((StyledText) e.widget).getText();
					setName(BotanicalName.PARSED_NAME(text));
				}
			}
		});

		// Listen for focus to send this name to the property sheet
		nameViewer.getTextWidget().addFocusListener(new FocusAdapter () {
			public void focusGained(FocusEvent e) {
				
				if (nameEditorView != null)
					nameEditorView.setPropertySheetNodes(getName());
			}
		});
		
		// Listen for user selecting text
		nameViewer.getTextWidget().addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}

			public void widgetSelected(SelectionEvent e) {
				// e.x = start, y = e.finish - strange but true
//				nameViewer.getAnnotationModel().addAnnotation(
//						new WarningAnnotation(0, "Could not parse name."), 
//						new Position(e.x, e.y - e.x));
			}
			
		});
		
		
		// Put name into name viewer, show error if parsed incorrectly
		nameViewer.getTextWidget().setText(getName().getTitleCache() == null ? "" : getName().getTitleCache());
		nameViewer.setShowError(getName().getHasProblem());
		
		// Put cursor at end of name viewer
		nameViewer.getTextWidget().setCaretOffset(
				nameViewer.getTextWidget().getText().length());
		
		// Set composite to initialized - any new input will now be parsed
		initialized = true;
		
		// Add drag functionality to synonyms only
		if (compositeType == ACCEPTED) return;
		
		// make menu temp
		createTempMenu();
		
		// **************** DRAG FUNCTIONALITY **************** //
		Transfer[] types = new Transfer[] {TaxonTransfer.getInstance()};
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;
		
		// Make composite draggable
		DragSource dragComposite = new DragSource (this, operations);
		dragComposite.setTransfer(types);
		dragComposite.addDragListener(dragSourceListener);

		// Make relation label draggable	
		DragSource dragLabel = new DragSource (relationLabel, operations);
		dragLabel.setTransfer(types);
		dragLabel.addDragListener(dragSourceListener);
	}
		
	/**
	 * Shared drag listener which destroys the NameComposite after the drag
	 *  gesture is completed.
	 */
	DragSourceListener dragSourceListener = new DragSourceListener () {
		
		CompositeRectangle rectangle;
		
		public void dragStart(DragSourceEvent event) {

			rectangle = new CompositeRectangle(NameComposite.this,
					Display.getDefault().getSystemColor(SWT.COLOR_GRAY));
			
			event.doit = true;
		}
		public void dragSetData (DragSourceEvent event) {
			TaxonTransfer.getInstance().setTaxon(synonym);
		}
		public void dragFinished(DragSourceEvent event) {
			if (event.detail == DND.DROP_MOVE) {
				Composite parent = NameComposite.this.getParent();
				Composite grandParent = parent.getParent();
				
//				rectangle.dispose();
				NameComposite.this.dispose();
				
				// If this was the last child in its homo. group,
				//  drop the group from the interface 
				if (parent.getChildren().length == 0)
					parent.dispose();
				else
					// Only used if we're dropping it on its current homo. group
					parent.layout();
				
				grandParent.layout();
			}
		}
	};
	private NameEditorView nameEditorView;
	
	public void setName(TaxonNameBase name) {
		(compositeType == ACCEPTED? taxon : synonym).setName(name);
		nameViewer.setShowError(name.getHasProblem());
	}
	
	public TaxonNameBase getName() {
		return (compositeType == ACCEPTED? taxon : synonym).getName();				
	}

	public void setNameEditorView(NameEditorView nameEditorView) {
		this.nameEditorView = nameEditorView;		
	}
	
	private void createTempMenu() {
		final Menu menu1 = new Menu(relationLabel);
		relationLabel.setMenu(menu1);
		relationLabel.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}

			public void mouseDown(MouseEvent e) {
				menu1.setVisible(true);
			}

			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			
		});

		final MenuItem deletehieraciumFuscoatrumMenuItem = new MenuItem(menu1, SWT.NONE);
		deletehieraciumFuscoatrumMenuItem.setText("Delete \"Hieracium wolffii subsp. trichotranssilvanicum\" from this taxon");

		new MenuItem(menu1, SWT.SEPARATOR);

		final MenuItem moveMenuItem = new MenuItem(menu1, SWT.NONE);
		moveMenuItem.setText("Make \"Hieracium wolffii subsp. trichotranssilvanicum\" the accepted name of a new taxon");

		final MenuItem makehieraciumFuscoatrumMenuItem = new MenuItem(menu1, SWT.NONE);
		makehieraciumFuscoatrumMenuItem.setText("Make \"Hieracium wolffii subsp. trichotranssilvanicum\" the accepted name of this taxon");

		new MenuItem(menu1, SWT.SEPARATOR);

		final MenuItem turnhieraciumFuscoatrumMenuItem_3 = new MenuItem(menu1, SWT.NONE);
		turnhieraciumFuscoatrumMenuItem_3.setText("Turn \"Hieracium wolffii subsp. trichotranssilvanicum\" into a basionym");

		final MenuItem turnhieraciumFuscoatrumMenuItem_4 = new MenuItem(menu1, SWT.NONE);
		turnhieraciumFuscoatrumMenuItem_4.setText("...");
	}

	
}