/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.controller;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeView;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;

/**
 * @author n.hoffmann
 * @created 20.01.2009
 * @version 1.0
 */
public class TreeController {
	private static final Logger logger = Logger.getLogger(TreeController.class);
	private static IViewPart treeView;

	/**
	 * Returns the <code>TaxonomicTreeViewer</code> used for navigation. Not to 
	 * be confused with the window that contains it, <code>TaxonomicTreeView</code>. 
	 * 
	 * @return
	 */
	public static TaxonomicTreeViewer getTreeViewer() {
		TaxonomicTreeView view = getTreeView();
		if (view == null) {
			return null;
		}
		return view.getTreeViewer();
	}

	/**
	 * @see eu.etaxonomy.taxeditor.navigation.TaxonomicTreeView#createTreeViewer()
	 * 
	 * @return
	 */
	public static TaxonomicTreeViewer createTreeViewer() {
		TaxonomicTreeView view = getTreeView();
		if (view == null) {
			return null;
		}
		return view.createTreeViewer();
	}

	/**
	 * Returns the <code>TaxonomicTreeView</code> containing the 
	 * <code>TaxonomicTreeViewer</code> used for navigation. 
	 * 
	 * @return
	 */	
	public static TaxonomicTreeView getTreeView() {
		if (treeView == null) {
			treeView = GlobalController.getViewById(TaxonomicTreeView.ID);
			if (!(treeView instanceof TaxonomicTreeView)) {
				treeView = null;
			}
		}
		return (TaxonomicTreeView) treeView;
	}
	
	
	/**
	 * Returns the selected taxon for referencing in context menus
	 * 
	 * @param event 
	 * @return
	 */
	public static Taxon getCurrentSelection(ExecutionEvent event){
		
		ISelection menuSelection = HandlerUtil.getActiveMenuSelection(event);
		
		// The selection should always be a tree selection since we are in the 
		// taxonomic tree view. Just in case this will be used in another spot.
		if(menuSelection instanceof TreeSelection){
			Taxon taxon = (Taxon) ((TreeSelection) menuSelection).getFirstElement();
			logger.debug("Selected taxon: " + taxon);
			return taxon;
		}
		return null;
	}
	
	
}
