/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;

/**
 * @author p.ciardelli
 * @created 16.05.2008
 * @version 1.0
 */
public class SimpleSelectionProvider implements ISelectionProvider {
	private static final Logger logger = Logger
			.getLogger(SimpleSelectionProvider.class);

	private ListenerList listeners = new ListenerList();
	private ISelection selection;
	
	public void addSelectionChangedListener(
			ISelectionChangedListener listener) {
		listeners.add(listener);
	}

	public ISelection getSelection() {
		if (selection != null)
			return selection;
		return null;
	}

	public void removeSelectionChangedListener(
			ISelectionChangedListener listener) {
		listeners.remove(listener);
	}

	public void setSelection(ISelection selection) {
		this.selection = selection;
		fireSelectionChanged();
	}

	private void fireSelectionChanged() {
		
		SelectionChangedEvent event = new SelectionChangedEvent(this, selection);
							
		for (Object listener : listeners.getListeners()) {
			 if (listener instanceof ISelectionChangedListener) {
				 ((ISelectionChangedListener)listener).selectionChanged(event);
			 }
		}
	}

}
