/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.handlers;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoableOperation;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.controller.TreeController;
import eu.etaxonomy.taxeditor.operations.DeleteTaxonOperation;

/**
 * @author n.hoffmann
 * @created 09.02.2009
 * @version 1.0
 */
public class DeleteTaxonHandler extends AbstractHandler implements IHandler {
	private static final Logger logger = Logger
			.getLogger(DeleteTaxonHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		Taxon taxon = TreeController.getCurrentSelection(event);
		
		IUndoableOperation operation;
		try {
			operation = new DeleteTaxonOperation(event.getCommand().getName(), 
					EditorController.getUndoContext(taxon), taxon);
			GlobalController.executeOperation(operation);
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}

		return null;
	}
}
