/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IFormPart;

import eu.etaxonomy.cdm.api.service.ILocationService;
import eu.etaxonomy.cdm.api.service.ILocationService.NamedAreaVocabularyType;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.location.TdwgArea;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.editor.reference.ReferenceSelectComposite;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 *
 */
public class DistributionDetailsPage extends AbstractDescriptionDetailsPage {
	private static final Logger logger = Logger.getLogger(DescriptionMasterDetailsBlock.class);


	/**
	 * Location service for retrieving 
	 */
	private ILocationService locationService;
	/**
	 * Number of area menus to show
	 */
	private static final int NUM_AREA_MENUS = 4; 
	/**
	 * UI elements
	 */
	private Combo vocabTypeCombo;
	private Combo[] areaCombos = new Combo[NUM_AREA_MENUS];
	private Combo statusCombo;
	private ReferenceSelectComposite referenceComposite;
	/**
	 * Distribution fields
	 */
	private NamedArea selectedArea;
	private PresenceAbsenceTermBase<?> selectedStatus;
	private NamedAreaVocabularyType selectedVocabType;
	private ReferenceBase<?> selectedReference;
	/**
	 * Lists to populate drop downs
	 */
	private ArrayList<NamedAreaVocabularyType> vocabularyTypes;  
	private List<PresenceAbsenceTermBase> statusTerms;
	private List<List<NamedArea>> areas = new ArrayList<List<NamedArea>>();
	private NamedArea[] selectedAreas = new NamedArea[NUM_AREA_MENUS];
	/**
	 * Static fields to persist default values
	 */
	private static ReferenceBase<?> lastReference;
	private static PresenceAbsenceTermBase<?> lastStatusTerm;
	private static NamedAreaVocabularyType lastVocabType;
	private static NamedArea lastNamedArea;
	Map<NamedAreaVocabularyType, List<NamedArea>> topLevelAreasByType = 
		new HashMap<NamedAreaVocabularyType, List<NamedArea>>();
	/**
	 * Used to disable listeners while input values are programmatically set
	 */
	private boolean selectionListenersEnabled;

	public DistributionDetailsPage() {
		
		locationService = CdmStore.getLocationService();
		
//		initTopLevelVocabularies();
	}

	/**
	 * There is no mechanism to retrieve a NamedArea's vocabulary type. Therefore, we
	 * store all top-level NamedAreas with their vocabulary types. 
	 */
//	private void initTopLevelVocabularies() {
//		for (NamedAreaVocabularyType vocabType : getVocabularyTypes()) {
//			List<NamedArea> list = new ArrayList<NamedArea>();
//			for (NamedArea topLevelArea : locationService.getTopLevelNamedAreasByVocabularyType(vocabType)) {
//				topLevelAreasByArea.put(topLevelArea, vocabType);
//				list.add(topLevelArea);
//			}
//			topLevelAreasByType.put(vocabType, list);
//		}
//	}
	
//	/**
//	 * Set inputs to selections from last time dialog was used.
//	 */
//	private void recallPreviousSelections() {
//		if (lastVocabType != null)  {
//			selectedVocabType = lastVocabType;
//
//			// Last named area don't make much sense w/out a vocab type
//			if (lastNamedArea != null) {
//				selectedArea = lastNamedArea;
//			}
//		}
//		if (lastStatusTerm != null) {
//			selectedStatus = lastStatusTerm; 
//		}
//		if (lastReference != null) {
//			selectedReference = lastReference;
//		}
//	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptionnew.AbstractDescriptionDetailsPage#selectionChanged(org.eclipse.ui.forms.IFormPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IFormPart part, ISelection selection) {
		super.selectionChanged(part, selection);
		
		setSelectionListenersEnabled(false);
		
		selectedArea = HibernateProxyHelper.deproxy(((Distribution) input).getArea(), NamedArea.class);
		populateAreas();
		
		populateVocabTypes();
		
		selectedStatus = HibernateProxyHelper.deproxy(((Distribution) input).getStatus(), PresenceAbsenceTermBase.class);
		populateStatus();
		
		setSelectionListenersEnabled(true);
	}
	
	/**
	 * @param enabled
	 */
	private void setSelectionListenersEnabled(boolean enabled) {
		this.selectionListenersEnabled = enabled;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptionnew.AbstractDescriptionDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) {
		super.createContents(parent);
		
		section.setText("Distribution Details"); //$NON-NLS-1$
		
        // create vocab type
		createLabel("Select an area vocabulary:", client, parent);
        vocabTypeCombo = createCombo(client);
        vocabTypeCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (selectionListenersEnabled) {
					selectVocabType();
				}
			}
		});
        
        // create areas
        createLabel("Select an area:", client, parent);
        for (int i = 0; i < NUM_AREA_MENUS; i++) {
        	areaCombos[i] = createCombo(client); 
            areaCombos[i].addSelectionListener(new AreaComboListener(i));
            areaCombos[i].setEnabled(false);
            areaCombos[i].setVisibleItemCount(20);
        }
        
        // create status
        createLabel("Select status:", client, parent);
		statusCombo =  new Combo(client, SWT.BORDER | SWT.READ_ONLY);
		statusCombo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
		statusCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (selectionListenersEnabled) {
					selectStatusTerm();
				}
			}
		});
		
		// create reference
        createCitationControls();

        populateVocabTypes();
		populateAreas();        
		populateStatus();
	}
	
	/**
	 * @return
	 */
	private Combo createCombo(Composite composite) {
		Combo combo = new Combo(composite, SWT.BORDER | SWT.READ_ONLY);
		combo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
				| GridData.HORIZONTAL_ALIGN_FILL));
		return combo;
	}
	
	class AreaComboListener extends SelectionAdapter {
    	private int i;
		AreaComboListener(int i) {
    		this.i = i;
    	}
		@Override
		public void widgetSelected(SelectionEvent e) {
			if (selectionListenersEnabled) {
				selectArea(i);				
			}
		}
    }
    
	/**
	 * @param selectedVocabType
	 * @return
	 */
	private List<NamedArea> getNamedAreas(NamedAreaVocabularyType vocabularyType) {
		List<NamedArea> topLevelTerms = getTopLevelAreas(vocabularyType);
	
		Collections.reverse(topLevelTerms);
		
		areas.add(0, topLevelTerms);
		return topLevelTerms;
	}
	
	/**
	 * @param vocabularyType
	 * @return
	 */
	private List<NamedArea> getTopLevelAreas(
			NamedAreaVocabularyType vocabularyType) {
		List<NamedArea> topLevelAreas = topLevelAreasByType.get(vocabularyType);
		if (topLevelAreas == null) {
			topLevelAreas = locationService.getTopLevelNamedAreasByVocabularyType(vocabularyType);
			topLevelAreasByType.put(vocabularyType, topLevelAreas);			
		}
		return topLevelAreas;
	}

	/**
	 * 
	 */
	private void populateVocabTypes() {
		
		vocabTypeCombo.removeAll();
		
		int i = 0;		
		int index = -1;
		for (NamedAreaVocabularyType vocabularyType : getVocabularyTypes()) {
			String vocabularyTypeLabel = vocabularyType == null ? 
					"" : vocabularyType.toString();
			vocabTypeCombo.add(vocabularyTypeLabel);
			
			if (vocabularyType != null && vocabularyType == selectedVocabType) {
				index = i;
			}
			i++;
		}
		if (index > -1) {
			vocabTypeCombo.select(index);
		}
	}

    /**
	 * 
	 */
	private void populateAreas() {
		if (selectedArea == null) {
			return;
		}
		
		for (int i = 0; i < NUM_AREA_MENUS; i++) {
			areaCombos[i].removeAll();
			areaCombos[i].setEnabled(false);
		}
		
		List<NamedArea> areasTmp = new ArrayList<NamedArea>();
		NamedArea namedAreaTmp = selectedArea;
		NamedArea parentArea = null;
		do {
			areasTmp.add(namedAreaTmp);
			parentArea = getPartOf(namedAreaTmp);
			if (parentArea == null) {
				selectedVocabType = getVocabularyType(namedAreaTmp);				
			}
			namedAreaTmp = parentArea;
		} while (parentArea != null);
		Collections.reverse(areasTmp);
		int depth = 0;
		for (NamedArea namedArea : areasTmp) {
			selectedAreas[depth] = namedArea;
			populateAreas(depth);
			depth++;
		}
	}
	
	/**
	 * @param depth
	 */
	private void populateAreas(int depth) {
				
		List<NamedArea> areasTmp;
		if(depth == 0){
		  	 areasTmp = getNamedAreas(selectedVocabType);
		}else{
			areasTmp = getNamedAreas(selectedAreas[depth - 1], depth);
		}
		
		int index = -1;
		int i = 0;
		if (areasTmp.size() > 0) {
			for(Object areaObject : areasTmp){
				NamedArea areaTmp = HibernateProxyHelper.deproxy(areaObject, NamedArea.class);
				String areaLabel = areaTmp == null ? "" : areaTmp.getLabel();
				areaCombos[depth].add(areaLabel);
				
				if (selectedAreas[depth] != null && selectedAreas[depth].equals(areaTmp)) {
					index = i;
				}
				i++;
			}
			areaCombos[depth].setEnabled(true);
			if (index > -1) {
				areaCombos[depth].select(index);
			}
		}
	}
	
	/**
	 * 
	 */
	private void populateStatus() {

		statusCombo.removeAll();
		
		int selectedIndex = -1;
		int i = 0;
		for (PresenceAbsenceTermBase<?> term : getStatusTerms()) {
			if (term == null) {
			} else {
				statusCombo.add(term.getLabel());		
				if (this.selectedStatus != null && this.selectedStatus.equals(term)) {
					selectedIndex = i;
				}
			}
			i++;
		}
		if (selectedIndex != -1) {
			statusCombo.select(selectedIndex);	
		}
	}

	/**
	 * 
	 */
	protected void selectVocabType() {
		int i = vocabTypeCombo.getSelectionIndex();
		selectedVocabType = getVocabularyTypes().get(i);
		clearAreas();
		
		populateAreas(0);
		
//		lastVocabType = selectedVocabType;
	}
	
	/**
	 * @param i
	 */
	protected void selectArea(int depth) {
		// This selection is made currently selected area
		int i = areaCombos[depth].getSelectionIndex();
		
		selectedArea = getSelectedArea(depth, i);
		
		selectedAreas[depth] = selectedArea;
		
		clearArea(depth + 1);
		populateAreas(depth + 1);
		
		setInputArea(selectedArea);
		
//		lastVocabType = selectedVocabType;
//		lastNamedArea = selectedArea;
	}

	/**
	 * 
	 */
	protected void selectStatusTerm() {
		int i = statusCombo.getSelectionIndex();
		selectedStatus = statusTerms.get(i);
		
		setInputStatus(selectedStatus);
		
//		lastStatusTerm = selectedStatus;
	}

	/**
	 * @param area
	 */
	private void setInputArea(NamedArea area) {
		((Distribution) input).setArea(area);
	}	
	
	/**
	 * @param status
	 */
	private void setInputStatus(PresenceAbsenceTermBase<?> status) {
		((Distribution) input).setStatus(status);
	}
	
	/**
	 * @param namedAreaTmp
	 * @return
	 */
	private NamedAreaVocabularyType getVocabularyType(NamedArea namedArea) {
		
		UUID tdwgUuid = VocabularyEnum.TdwgArea.getUuid();
		UUID waterbodyOrCountryUuid = VocabularyEnum.WaterbodyOrCountry.getUuid();
		UUID continentUuid = VocabularyEnum.Continent.getUuid();
		
		TermVocabulary<NamedArea> vocabulary = HibernateProxyHelper.deproxy(namedArea.getVocabulary(), TermVocabulary.class);
		UUID vocabularyUuid = vocabulary.getUuid();
		
		if (vocabularyUuid.equals(tdwgUuid)) {
			return NamedAreaVocabularyType.TDWG_AREA;			
		}

		if (vocabularyUuid.equals(waterbodyOrCountryUuid)) {
			return NamedAreaVocabularyType.WATERBODY_OR_COUNTRY;			
		}
		
		if (vocabularyUuid.equals(continentUuid)) {
			return NamedAreaVocabularyType.CONTINENT;			
		}		
		
		logger.warn("No matching vocabulary type found for NamedArea " + namedArea + ".");
		return null;
	}

	/**
	 * @return
	 */
	private List<NamedAreaVocabularyType> getVocabularyTypes() {
		if (vocabularyTypes == null) {
			vocabularyTypes = new ArrayList<NamedAreaVocabularyType>();
			vocabularyTypes.addAll(locationService.getNamedAreaVocabularyTypes());
		}
		return vocabularyTypes;
	}
	
	/**
	 * @param namedArea
	 * @return
	 */
	private List<NamedArea> getNamedAreas(NamedArea namedArea, int depth) {
		List<NamedArea> terms = new ArrayList<NamedArea>();
		terms.addAll(namedArea.getIncludes());
		
		areas.add(depth, terms);
		
		return terms;
	}
	
	/**
	 * @return
	 */
	private List<PresenceAbsenceTermBase> getStatusTerms() {
		if (statusTerms == null) {
			statusTerms = new ArrayList<PresenceAbsenceTermBase>();
			for (PresenceAbsenceTermBase statusTerm : PreferencesUtil.getPreferredPresenceAbsenceTerms()) {
				statusTerms.add(statusTerm);
			}
		}
		return statusTerms;
	}
	
	/**
	 * @return
	 */
	private NamedAreaVocabularyType getVocabType() {
		int i = vocabTypeCombo.getSelectionIndex();
		return vocabularyTypes.get(i);
	}

	/**
	 * @param i
	 * @param index
	 * @return
	 */
	private NamedArea getSelectedArea(int depth, int index) {
		return areas.get(depth).get(index);
	}
	
	/**
	 * @return
	 */
	private PresenceAbsenceTermBase<?> getStatusTerm() {
		int i = statusCombo.getSelectionIndex();
		return statusTerms.get(i);
	}
	
	/**
	 * @param namedArea
	 * @return
	 */
	private NamedArea getPartOf(NamedArea namedArea) {
		if (namedArea instanceof TdwgArea) {
			return HibernateProxyHelper.deproxy(((NamedArea) namedArea).getPartOfWorkaround(), TdwgArea.class);
		}
		return null;
	}
	
	/**
	 * @param i
	 */
	private void clearArea(int i) {
		for (int j = i; j < NUM_AREA_MENUS; j++) {
			// TODO add clear selectedArea
			areaCombos[j].removeAll();
			areaCombos[j].setEnabled(false);
		}
	}

	/**
	 * 
	 */
	private void clearAreas() {
		clearArea(0);
	}
}
