/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import java.awt.Color;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.IFormPart;

import eu.etaxonomy.cdm.ext.EditGeoServiceUtilities;
import eu.etaxonomy.cdm.ext.IEditGeoService;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 *
 */
public class DistributionMapDetailsPage extends AbstractDescriptionDetailsPage {
	private static final Logger logger = Logger
			.getLogger(DistributionMapDetailsPage.class);
	
	private Browser browser;
	private IEditGeoService geoService;

	private Composite parent;

	private TaxonDescription description;
	
	public DistributionMapDetailsPage() {
		geoService = CdmStore.getGeoService();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptiontree.AbstractDescriptionDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createContents(Composite parent) {
		super.createContents(parent);
		
		this.parent = parent;
		
		section.setText("Distribution Overview"); //$NON-NLS-1$
		
//		GridLayout gridLayout = new GridLayout();
//		gridLayout.numColumns = 1;
//		shell.setLayout(gridLayout);
		
		browser = new Browser(client, SWT.NONE);
		
		GridData data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		data.verticalAlignment = GridData.FILL;
		data.grabExcessHorizontalSpace = true;
		data.grabExcessVerticalSpace = true;
		data.heightHint = 300;
		data.widthHint = 500;
		
		browser.setLayoutData(data);
		
//		final ProgressBar progressBar = new ProgressBar(client, SWT.NONE);
//		data = new GridData();
//		data.horizontalAlignment = GridData.END;
//		progressBar.setLayoutData(data);

//		browser.addProgressListener(new ProgressListener() {
//			public void changed(ProgressEvent event) {
//					if (event.total == 0) return;                            
//					int ratio = event.current * 100 / event.total;
//					progressBar.setSelection(ratio);
//			}
//			public void completed(ProgressEvent event) {
//				progressBar.setSelection(0);
//			}
//		});
		
//		// Create "Refresh Map" button
//		final Button refreshButton = mform.getToolkit().createButton(client, "Refresh Map", SWT.PUSH);
//		refreshButton.setLayoutData(new GridData());
//		refreshButton.addMouseListener(new MouseAdapter() {
//			public void mouseUp(MouseEvent e) {
//				loadMap();
//			}
//		});	
		
		parent.addControlListener(new ControlListener(){
			public void controlMoved(ControlEvent e) {
				// do nothing
			}
			public void controlResized(ControlEvent e) {
				loadMap();
			}
		});
	}
	
	private void loadMap(){
		if (description != null) {
			final String url = getMapUrl();
			
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					browser.setUrl(url);
				}
			});
		}
	}
	
	private String getMapUrl(){
		// FIXME make these hardcoded parameters configurable via preferences
		String url = "http://edit.csic.es/v1/areas.php";
		Map<PresenceAbsenceTermBase<?>,Color> presenceAbsenceTermColors = null;
		int width = calculateWidth();
		int height = 0; 
		String bbox = "-180,-90,180,90"; 
		String backLayer = null;

		String parameter = EditGeoServiceUtilities.getEditGeoServiceUrlParameterString(getDistributions(),
				presenceAbsenceTermColors, 0, 0, null, "tdwg4");
		
		return url + "?" + parameter + "&bbox=-180,-90,180,90&ms=400";
	}

	/**
	 * @return
	 */
	private Set<Distribution> getDistributions() {
		HashSet<Distribution> distributions = new HashSet<Distribution>();
		for (DescriptionElementBase element : description.getElements()) {
			if (element instanceof Distribution) {
				distributions.add((Distribution) element);
			}
		}
		return distributions;
	}

	/**
	 * @return
	 */
	private int calculateWidth(){
		return parent.getSize().x - 10;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptiontree.AbstractDescriptionDetailsPage#selectionChanged(org.eclipse.ui.forms.IFormPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IFormPart part, ISelection selection) {
		super.selectionChanged(part, selection);
		description = ((TaxonDescriptionFeature) input).getDescription();
		loadMap();
	}
}
