/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.model.NameHelper;
import eu.etaxonomy.taxeditor.operations.ChangeHomotypicGroupOperation;
import eu.etaxonomy.taxeditor.operations.ChangeSynonymToMisapplicationOperation;
import eu.etaxonomy.taxeditor.propertysheet.name.SynonymPropertySource;

/**
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class SynonymComposite extends NameComposite<Synonym> {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(SynonymComposite.class);
	
	public SynonymComposite(TaxonNameEditor editor, Composite parent, Synonym synonym) {
		super(editor, parent, synonym);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#initializeComposite()
	 */
	@Override
	protected void initializeComposite() {
		setIsDraggable(true);
		setFont(getViewerFont());
		setIndent(SYNONYM_INDENT);
		
		if (isHomotypic()) {
			if (NameHelper.isNameGroupBasionym(getSynonym().getName())) {
				setIcon(HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HOMOTYPIC_SYNONYM_ICON);
			}	
		} else {
			if (NameHelper.isNameGroupBasionym(getSynonym().getName())) {
				setIcon(HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HETEROTYPIC_SYNONYM_ICON);
			}
		}
		
		initTextViewer(getSynonym());
	}
	
	private boolean isHomotypic() {
		HomotypicalGroup group = getSynonym().getHomotypicGroup();
		return group.equals(getEditorTaxon().getHomotypicGroup()); 
	}
	
	public Synonym getSynonym() {
		return getTaxonBase();
	}

	public boolean setParent(Composite parent) {

		boolean doSetParent = false;			
			
		// Has this been moved to the misapplied names group?
		if (parent instanceof MisappliedGroupComposite) {
			IUndoableOperation operation = new ChangeSynonymToMisapplicationOperation
					("change to misapplication", editor.getUndoContext(), getEditorTaxon(), getSynonym(), editor); //$NON-NLS-1$
			EditorUtil.executeOperation(operation);
			
			doSetParent = true;
		}

		// Has this been moved to a HomotypicalGroup?
		if (parent instanceof HomotypicalGroupComposite) {
			
			HomotypicalGroup homotypicalGroup = 
					((HomotypicalGroupComposite)parent).getGroup();
			
			// Make sure we are not dropping synonym on its own group
			if (!homotypicalGroup.equals(getSynonym().getHomotypicGroup())) {
				IUndoableOperation operation = new ChangeHomotypicGroupOperation
						("change type", editor.getUndoContext(), getEditorTaxon(), getSynonym(), homotypicalGroup, editor); //$NON-NLS-1$
				EditorUtil.executeOperation(operation);
				
				doSetParent = true;
			}
		}
		return doSetParent;
	}
		
	public IPropertySource getPropertySource() {
		if (NameHelper.isNameGroupBasionym(getSynonym().getName())) {
			return new BasionymSynonymPropertySource(getSynonym());
		} else {
			return new SynonymPropertySource(getSynonym());
		}
	}

	protected Font getViewerFont() {
		return SYNONYM_FONT;
	}

	@Override
	public TaxonNameBase<?, ?> getName() {
		return getSynonym().getName();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#setName(eu.etaxonomy.cdm.model.name.TaxonNameBase)
	 */
	@Override
	public void setName(TaxonNameBase name) {
		getSynonym().setName(name);	
	}
	
	public Object getData () {
		return getSynonym();
	}
}