/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredAgentSelectionDialog;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredTeamOrPersonBaseSelectionDialog;

/**  
 * @author p.ciardelli
 * @created 19.05.2008
 * @version 1.0
 */
public class AuthorshipSearchDescriptor extends PropertyDescriptor {
			
	private TeamOrPersonBase teamOrPerson;

	public AuthorshipSearchDescriptor(Object id, String displayName) {
		super(id, displayName);
	}

    /**
	 * @param idAuthorshipcache
	 * @param authorshipcache
	 * @param combinationAuthorTeam
	 */
	public AuthorshipSearchDescriptor(Object id, String displayName, TeamOrPersonBase teamOrPerson) {
		this(id, displayName);
		this.teamOrPerson = teamOrPerson;
	}

	public CellEditor createPropertyEditor(Composite parent) {
        CellEditor editor = new DialogCellEditor(parent) {

			/* (non-Javadoc)
			 * @see org.eclipse.jface.viewers.DialogCellEditor#openDialogBox(org.eclipse.swt.widgets.Control)
			 */
			protected Object openDialogBox(
					Control cellEditorWindow) {
				AgentBase agent = FilteredAgentSelectionDialog.selectAgent(cellEditorWindow.getShell(), teamOrPerson);
//				AgentBase agent = FilteredTeamOrPersonBaseSelectionDialog.selectTeamOrPersonBase(cellEditorWindow.getShell(), teamOrPerson);
				if (agent instanceof INomenclaturalAuthor) {
					return agent;
				} else {
					MessageDialog.openError(cellEditorWindow.getShell(), "Not a nomenclatural author", 
												"'" + agent.getTitleCache() + "' is not a nomenclatural author.");
				}
				return null;
			}
        };
        if (getValidator() != null) {
			editor.setValidator(getValidator());
		}
        return editor;
    }
}
