/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.propertysheet.namerelations.wizard.NameRelationsListWizard;

/**
 * Property sheet descriptor that pops up a dialog window
 *  returning a set.
 *  
 * @author p.ciardelli
 * @created 19.05.2008
 * @version 1.0
 */
public class NameRelationsPropertyDescriptor extends PropertyDescriptor {
	private static final Logger logger = Logger
			.getLogger(NameRelationsPropertyDescriptor.class);
	
	private TaxonNameBase name;
	
	public NameRelationsPropertyDescriptor(Object id, String displayName, TaxonNameBase name) {
		super(id, displayName);
		this.name = name;
	}

    public CellEditor createPropertyEditor(Composite parent) {
        CellEditor editor = new DialogCellEditor(parent) {
			
			@Override
			protected Object openDialogBox(
					Control cellEditorWindow) {
				Wizard wizard = new NameRelationsListWizard(name);
				WizardDialog dialog = new WizardDialog(EditorUtil.getShell(), wizard);
				dialog.create();
				int returnCode = dialog.open();
				
				if (returnCode == Window.OK) {
					return new NameRelationsPropertySource(name);
				} else {
					return null;
				}
				
			}
        };
        if (getValidator() != null) {
			editor.setValidator(getValidator());
		}
        return editor;
    }
}