// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created 06.07.2009
 * @version 1.0
 */
public class DatasourceLauncherDialog extends TitleAreaDialog {


	private static final Logger logger = Logger
			.getLogger(DatasourceLauncherDialog.class);
	private Combo combo_dataSources;
	private int selection;
	private List<ICdmDataSource> dataSources;
	private Button okButton;
	private Button button_toggle;
	
	/**
	 * @param parentShell
	 */
	public DatasourceLauncherDialog(Shell parentShell) {
		super(parentShell);
		
		dataSources = CdmDataSourceRepository.getAll();
		selection = -1;
		String lastUsedDataSourceName = CdmDataSourceRepository.getLastUsedDataSourceName();
		for(ICdmDataSource dataSource : dataSources){
			selection++;
			if(dataSource.getName().equals(lastUsedDataSourceName)){
				break;
			}
		}

		
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
	    Control contents = super.createContents(parent);

		setTitle("Choose Datasource");
		setMessage("Choose a data source to connect to");
		
		this.getShell().setText("Datasource Launcher");
		
	    return contents;
	  }

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
        Composite container = (Composite) super.createDialogArea(parent);
        
        // create composite
        Composite composite = new Composite(container, SWT.NULL);
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        composite.setLayout(layout);
        
        final CLabel label_dataSources = new CLabel(composite, SWT.NULL); 
        label_dataSources.setText("Datasource:");
        
		combo_dataSources = new Combo(composite, SWT.DROP_DOWN | SWT.READ_ONLY);
		combo_dataSources.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
		int i = 0;
		
		for (ICdmDataSource dataSource : dataSources){
			String name = dataSource.getName() != null ? dataSource.getName() : "datasource name not set";
			String server = dataSource.getServer() != null ? dataSource.getServer() : "local";
			String dataBase = dataSource.getDatabase() != null ? dataSource.getDatabase() : "database name not set";
			
			combo_dataSources.add(name + " [" + server + " - " + dataBase + "]", i++);
		}
		
		combo_dataSources.addSelectionListener(new SelectionAdapter(){

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				selection = combo_dataSources.getSelectionIndex();
			}			
		});
		
		combo_dataSources.select(selection);
		
		Composite toggle = new Composite(composite, SWT.NULL);
		toggle.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		toggle.setLayout(new RowLayout(SWT.HORIZONTAL));
		
		button_toggle = new Button(toggle, SWT.CHECK);		
		
		CLabel label_info = new CLabel(toggle, SWT.NULL);
		label_info.setText("Use this as the default and do not ask me again (may be configured in Preferences)");
		
		button_toggle.addMouseListener(new MouseAdapter(){

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.MouseAdapter#mouseDown(org.eclipse.swt.events.MouseEvent)
			 */
			@Override
			public void mouseUp(MouseEvent e) {
				PreferencesUtil.getPreferenceStore().setValue(PreferencesUtil.HIDE_DATASOURCE_CHOOSER, true);
			}
		});
		
        applyDialogFont(composite);
        return composite;
	}

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
     */
    protected void createButtonsForButtonBar(Composite parent) {
        // create OK and Cancel buttons by default
        okButton = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
        //do this here because setting the text will set enablement on the ok
        // button
        combo_dataSources.setFocus();
    }
    
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	@Override
	protected void okPressed() {
		super.okPressed();
		ICdmDataSource chosenDataSource = dataSources.get(selection);		
		CdmDataSourceRepository.setCurrentDataSource(chosenDataSource);
	}
}
