// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs.filteredSelection;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.parser.ParserUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public class FilteredNameSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog<TaxonNameBase> {
	private static final Logger logger = Logger
	.getLogger(FilteredNameSelectionDialog.class);

	/**
	* @param taxon
	* @return
	*/
	public static TaxonNameBase selectName(Shell shell, TaxonNameBase name) {
		FilteredNameSelectionDialog dialog = new FilteredNameSelectionDialog(shell, 
				"Choose a name", false, name);
		return getSelectionFromDialog(dialog);
	}
	
	/**
	* @param shell
	* @param title
	 * @param name 
	*/
	protected FilteredNameSelectionDialog(Shell shell, String title, boolean multi, TaxonNameBase name) {
		super(shell, title, multi, FilteredNameSelectionDialog.class.getCanonicalName(), name);
	}
	
	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	*/
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		Link link = new Link(parent, SWT.NONE);
		link.setText("Click <A>here</A> to create a new name.");
		link.addListener (SWT.Selection, new Listener () {
			public void handleEvent(Event event) {
				InputDialog dialog = new InputDialog(getShell(), "Create a name", "Enter new name", "", null);
				if (dialog.open() == Window.OK) {
					
					TaxonNameBase name = ParserUtil.parseReferencedName(dialog.getValue());
					
					addObjectToModel(name);
					setPattern(name);
				}
			}
		});
		return link;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected TaxonNameBase getPersistentObject(UUID cdmUuid) {
		return CdmStore.getNameService().getTaxonNameByUuid(cdmUuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {
		model = CdmStore.getNameService().getUuidAndTitleCache();		
	}
}
