/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;

import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.preference.wizards.VocabularyTermWizard;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author p.ciardelli
 * @created 17.09.2008
 * @version 1.0
 * @author n.hoffmann
 */
public class FeatureMenuPreferences extends AbstractMenuPreferences<Feature> {

	public static final String PLUGIN_ID = "eu.etaxonomy.taxeditor.preferences.feature";
	
	public FeatureMenuPreferences() {
		super(VocabularyStore.getFeatures(), 
				"Feature Preferences", 
				"Choose which features you would like to use for descriptive elements.", 
				true);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.preference.AbstractMenuPreferences#addEditButtonListeners()
	 * 
	 * TODO refactor into superclass
	 */
	@Override
	protected void addNewButtonListeners() {
		newButton.addMouseListener(new MouseAdapter(){

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events.MouseEvent)
			 */
			@Override
			public void mouseUp(MouseEvent e) {
				super.mouseUp(e);
				VocabularyTermWizard<Feature> wizard = new VocabularyTermWizard<Feature>(VocabularyEnum.Feature); 
				
				WizardDialog dialog = new WizardDialog(StoreUtil.getShell(), wizard);
			    dialog.open();
			    
			    tableViewer.setInput(VocabularyStore.getFeatures());
			}
			
		});
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.preference.AbstractMenuPreferences#addEditButtonListeners()
	 */
	@Override
	protected void addEditButtonListeners() {
		editButton.addMouseListener(new MouseAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events.MouseEvent)
			 */
			@Override
			public void mouseUp(MouseEvent e) {
				super.mouseUp(e);
				
				Feature selection = (Feature) ((StructuredSelection) tableViewer.getSelection()).getFirstElement();
				
				VocabularyTermWizard<Feature> wizard = new VocabularyTermWizard<Feature>(VocabularyEnum.Feature, selection); 
				
				WizardDialog dialog = new WizardDialog(StoreUtil.getShell(), wizard);
			    dialog.open();
			    
			    tableViewer.setInput(VocabularyStore.getFeatures());
			    tableViewer.reveal(selection);
			}
		});
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.preference.AbstractMenuPreferences#addEditButtonListeners()
	 * 
	 * TODO refactor into superclass
	 */
//	@Override
//	protected void addDeleteButtonListeners() {
//		removeButton.addMouseListener(new MouseAdapter(){
//
//			/* (non-Javadoc)
//			 * @see org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events.MouseEvent)
//			 */
//			@Override
//			public void mouseUp(MouseEvent e) {
//				super.mouseUp(e);
//
//				Feature selected = (Feature) ((StructuredSelection) tableViewer.getSelection()).getFirstElement();
//			    
//				VocabularyStore.delete(selected);
//				
//				
//			    tableViewer.setInput(VocabularyStore.getFeatures());
//			}
//			
//		});
//	}
}
